<?php
/**
 * @file The HostingDistribution class.
 */

class HostingDistribution {

  use HostingDistributionNode;

  // The distribution node.
  public $node = FALSE;

  // The distribution's upgrade target.
  private $upgrade_target = FALSE;

  function __construct($node = FALSE) {
    if ($node && $node->type == 'distribution') {
      $this->node = $node;
    }

    $this->log = new HostingDistributionLog($this);
  }

  /**
   * Return the specified upgrade target (platform) for this distribution.
   */
  public function getUpgradeTarget() {
    if (!$this->upgrade_target) {
      $this->upgrade_target = $this->getEntityReference('field_upgrade_target');
    }
    return $this->upgrade_target;
  }

  /**
   * Return distribution-related menu items.
   */
  static function getMenuItems() {
    $items = array();

    $items['hosting/distributions/list'] = array(
      'title' => 'List',
      'type' => MENU_DEFAULT_LOCAL_TASK,
      'weight' => -10,
    );
    $items['hosting/distributions/add'] = array(
      'title' => 'Add distribution',
      'type' => MENU_LOCAL_TASK,
      'page callback' => 'drupal_goto',
      'page arguments' => array('node/add/distribution'),
      'access callback' => 'node_access',
      'access arguments' => array('create', 'distribution'),
    );
    $items['hosting_confirm/%/%/compare'] = array(
      'title' => 'Compare package upgrades',
      'page callback' => 'hosting_distribution_upgrade_comparison',
      'access callback' => 'hosting_distribution_task_access',
      'access arguments' => array(1),
      'type' => MENU_CALLBACK,
    );
    return $items;
  }

  /**
   * Alter the display of a node.
   */
  public function nodeView() {
    $this->fixNodeFieldDisplay('field_upgrade_target');
  }

}
