<?php
/**
 * @file The HostingDistributionUpgrade class.
 */

class HostingDistributionPlatform {

  use HostingDistributionNode;

  // The platform node.
  public $node = FALSE;

  // The platform's distribution.
  private $distribution = FALSE;

  // The platform's upgrade target.
  private $upgrade_target = FALSE;

  public function __construct($node = FALSE) {
    if ($node && $node->type == 'platform') {
      $this->node = $node;
    }
  }

  public function getDistribution() {
    if (!$this->distribution) {
      $node = node_load($this->getEntityReference('field_distribution'));
      $this->distribution = new HostingDistribution($node);
    }
    return $this->distribution;
  }

  public function getUpgradeTarget() {
    if (!$this->upgrade_target) {
      $this->upgrade_target = $this->getDistribution()->getUpgradeTarget();
    }
    return $this->upgrade_target;
  }

  /**
   * Access callback to ensure an upgrade target is specified, the site or
   * platform is enabled, and user has appropriate permission.
   */
  public function taskAccess() {
    $target = $this->getUpgradeTarget();

    $site_count = hosting_site_count($this->node->nid);

    if (!$target || $target === $this->node->nid || !$site_count) {
      return FALSE;
    }
    return $this->node->status && user_access("administer platforms") || user_access("create platform upgrade task");
  }

  /**
   * Alter the display of a node.
   */
  public function nodeView() {
    $this->fixNodeFieldDisplay('field_distribution');
  }

}
